@@ -3,7 +3,7 @@ package Module::Install::XSUtil;
 
 use 5.005_03;
 
-$VERSION = '0.35';
+$VERSION = '0.37';
 
 use Module::Install::Base;
 @ISA     = qw(Module::Install::Base);
@@ -26,7 +26,7 @@ my %BuildRequires = (
 );
 
 my %Requires = (
-    'XSLoader' => 0.10, # the newer, the better
+    'XSLoader' => 0.02,
 );
 
 my %ToInstall;
@@ -67,7 +67,7 @@ sub _xs_initialize{
                 $self->makemaker_args->{OPTIMIZE} = '-Zi';
             }
             else{
-                $self->makemaker_args->{OPTIMIZE} = '-g';
+                $self->makemaker_args->{OPTIMIZE} = '-g -ggdb -g3';
             }
             $self->cc_define('-DXS_ASSERT');
         }
@@ -97,9 +97,13 @@ sub _is_msvc{
 
     my $want_xs;
     sub want_xs {
-        my $default = @_ ? shift : 1; # you're using this module, you /must/ want XS by default
+        my($self, $default) = @_;
         return $want_xs if defined $want_xs;
 
+        # you're using this module, you must want XS by default
+        # unless PERL_ONLY is true.
+        $default = !$ENV{PERL_ONLY} if not defined $default;
+
         foreach my $arg(@ARGV){
             if($arg eq '--pp'){
                 return $want_xs = 0;
@@ -232,7 +236,7 @@ C99
 
     $tmpfile->close();
 
-    system $Config{cc}, '-c', $tmpfile->filename;
+    system "$Config{cc} -c " . $tmpfile->filename;
 
     (my $objname = File::Basename::basename($tmpfile->filename)) =~ s/\Q.c\E$/$Config{_o}/;
     unlink $objname or warn "Cannot unlink $objname (ignored): $!";
@@ -769,4 +773,4 @@ sub const_cccmd {
 1;
 __END__
 
-#line 980
+#line 984
